#include "itkImage.h"
#include "itkGDCMImageIO.h"
#include "itkGDCMSeriesFileNames.h"
#include "itkImageSeriesReader.h"

int
main(int argc, char * argv[])
{
  if (argc < 2)
  {
	std::cerr << "Usage: " << std::endl;
	std::cerr << argv[0] << " [DicomDirectory  [outputFileName  [seriesName]]]";
	std::cerr << "\nIf DicomDirectory is not specified, current directory is used\n";
  }
  std::string dirName = "."; // current directory by default
  if (argc > 1)
  {
	dirName = argv[1];
  }

  typedef float InputPixelType;
  constexpr unsigned int dimension = 3;
	
  // Set up the input image type, reader type & GDCM IO.
  typedef itk::Image<InputPixelType, dimension> InputImageType;
  typedef itk::ImageSeriesReader<InputImageType> ReaderType;

  typedef itk::GDCMImageIO ImageIOType;
  ImageIOType::Pointer gdcmIO = ImageIOType::New();
	
  using NamesGeneratorType = itk::GDCMSeriesFileNames;
  auto nameGenerator = NamesGeneratorType::New();

  nameGenerator->SetUseSeriesDetails(true);
  nameGenerator->AddSeriesRestriction("0008|0021");
  nameGenerator->SetGlobalWarningDisplay(false);
  nameGenerator->SetDirectory(dirName);

  try
  {
	  const typename ReaderType::FileNamesContainer fileNames = nameGenerator->GetInputFileNames();
	  
	  typename ReaderType::Pointer reader = ReaderType::New();
	  reader->SetImageIO(gdcmIO);
	  reader->SetFileNames(fileNames);
	  reader->Update();
	  
	  //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	  //	Examine file(s)
	  //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	  
	  typedef itk::ImageRegionIterator<InputImageType> ImageRegionIteratorType;
	  typedef itk::ImageRegionConstIterator<InputImageType> ConstImageRegionIteratorType;
	  
	  typename InputImageType::Pointer image = reader->GetOutput();
	  
	  const typename InputImageType::RegionType& region = image->GetRequestedRegion();
	  const typename InputImageType::SpacingType& spacing = image->GetSpacing();
	  const typename InputImageType::SizeType size = region.GetSize();
	  
	  const typename InputImageType::PointType& origin = image->GetOrigin();
	  
	  std::cout << "\n";
	  std::cout << "\nOrigin = " << origin << "\nSpacing = " << spacing << "\nSize = " << size << "\n";
	  std::cout << std::endl;
  }
  catch (const itk::ExceptionObject & ex)
  {
	std::cout << ex << std::endl;
	return EXIT_FAILURE;
  }
  return EXIT_SUCCESS;
}
